/*
 * Binary tree database functions for walking through all values.
 *
 * Copyright 2003 Andrew Wood, distributed under the Artistic License.
 */

#include "config.h"
#include "database.h"

#ifdef USING_INTERNAL_DB

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "btreei.h"


/*
 * Return the "first" key in the database, suitable for using with repeated
 * calls to qdb_nextkey() to walk through every key in the database.
 */
qdb_datum qdb_firstkey(qdb_t db)
{
	struct bt_record record;
	qdb_datum key;

	key.data = NULL;
	key.size = 0;

	if (fseek(db->fptr, 2 * sizeof(unsigned long), SEEK_SET) < 0)
		return key;

	if (fread(&record, BT_RECORD_SIZE, 1, db->fptr) < 1) {
		return key;
	}

	record.token[BT_TOKEN_MAX - 1] = 0;

	key.data = strdup(record.token);
	key.size = strlen(record.token);

	return key;
}


/*
 * Return the "next" key in the database, or key.data=NULL when all keys
 * have been returned.
 */
qdb_datum qdb_nextkey(qdb_t db, qdb_datum key)
{
	struct bt_record record;
	unsigned long offset, parent;
	qdb_datum newkey;

	newkey.data = NULL;
	newkey.size = 0;

	if (key.data == NULL) {
		return newkey;
	}

	dbbt_find_token(db, key, &record, &offset, &parent);

	if (offset < 1) {
		return newkey;
	}

	if (fseek(db->fptr, offset + BT_RECORD_SIZE, SEEK_SET) < 0) {
		return newkey;
	}

	do {
		if (fread(&record, BT_RECORD_SIZE, 1, db->fptr) < 1) {
			return newkey;
		}
	} while (record.lower & 0x80000000);

	record.token[BT_TOKEN_MAX - 1] = 0;

	newkey.data = strdup(record.token);
	newkey.size = strlen(record.token);

	return newkey;
}

#endif				/* USING_INTERNAL_DB */

/* EOF */
